package com.mcwl.web.controller.pay.AliPay;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.db.sql.Order;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.alipay.easysdk.factory.Factory;
import com.alipay.easysdk.kernel.Config;
import com.alipay.easysdk.payment.facetoface.models.AlipayTradePrecreateResponse;
import com.mcwl.common.core.redis.RedisCache;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.memberCenter.domain.Member;
import com.mcwl.memberCenter.domain.MemberLevel;
import com.mcwl.memberCenter.service.MemberLevelService;
import com.mcwl.memberCenter.service.MemberService;
import com.mcwl.pay.domain.OrderTrade;
import com.mcwl.pay.domain.OrderTradeDto;
import com.mcwl.resource.domain.MallProduct;
import com.mcwl.resource.service.MallProductService;
import com.mcwl.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 支付宝支付
 */
@Component

public class AliPayIntegration {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private MemberService memberService;

    @Autowired
    private MemberLevelService memberLevelService;

    @Autowired
    private MallProductService mallProductService;

    @Autowired
    private ISysUserService sysUserService;


    public AliPayIntegration(Config config) {
        Factory.setOptions(config);
    }


    /**
     * 调用支付宝预下订单接口
     *
     * @param orderTradeDto 订单实体
     * @return 二维码url
     * @throws Exception
     */
    public String orderPay(OrderTradeDto orderTradeDto) throws Exception {
        Integer productId = orderTradeDto.getProductId();
        if (!Optional.ofNullable(productId).isPresent()) {
            throw new ServiceException("mallProductId不能为空");
        }

        MallProduct mallProduct = mallProductService.getById(productId);
        if (!Optional.ofNullable(mallProduct).isPresent()) {
            throw new ServiceException("mallProduct不存在");
        }

        // 设置orderTrade信息
        OrderTrade tradeEntity = new OrderTrade();
        BeanUtil.copyProperties(orderTradeDto, tradeEntity);
        tradeEntity.setCode(UUID.randomUUID().toString(true).substring(0, 30));
        tradeEntity.setUserId(SecurityUtils.getUserId());
        tradeEntity.setUserName(SecurityUtils.getUsername());
        tradeEntity.setProductName(mallProduct.getProductName());

        //调用支付宝的接口
        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace()
                .preCreate(mallProduct.getProductName(),
                        tradeEntity.getCode() + "_product",
                        orderTradeDto.getPaymentAmount().toString());
        // 缓存到redis
        redisCache.setCacheObject(tradeEntity.getCode() + "_product", JSONUtil.toJsonStr(tradeEntity), 3, TimeUnit.MINUTES);
//        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace().preCreate("订单主题：Mac笔记本", "LS123qwe123", "19999");
        //参照官方文档响应示例，解析返回结果
        String httpBodyStr = payResponse.getHttpBody();
        JSONObject jsonObject = JSONObject.parseObject(httpBodyStr);
        return jsonObject.getJSONObject("alipay_trade_precreate_response").get("qr_code").toString();
    }


    /**
     * 调用支付宝预下订单接口
     *
     * @param orderTradeDto 订单实体
     * @return 二维码url
     * @throws Exception
     */
    public String memberPay(OrderTradeDto orderTradeDto) throws Exception {

        // 会员等级id
        Integer productId = orderTradeDto.getProductId();

        if (!Optional.ofNullable(productId).isPresent()) {
            throw new ServiceException("memberLevelId不能为空");
        }

        MemberLevel memberLevel = memberLevelService.getById(productId);

        if (!Optional.ofNullable(memberLevel).isPresent()) {
            throw new ServiceException("memberLevel不存在");
        }

        // 设置orderTrade信息
        OrderTrade tradeEntity = new OrderTrade();
        BeanUtil.copyProperties(orderTradeDto, tradeEntity);
        tradeEntity.setCode(UUID.randomUUID().toString(true).substring(0, 30));
        tradeEntity.setUserId(SecurityUtils.getUserId());
        tradeEntity.setUserName(SecurityUtils.getUsername());
        tradeEntity.setProductName(memberLevel.getMemberName());

        //调用支付宝的接口
        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace()
                // 设置过期时
                .preCreate(memberLevel.getMemberName(),
                        tradeEntity.getCode() + "_member",
                        orderTradeDto.getPaymentAmount().toString());
        // 缓存到redis
        redisCache.setCacheObject(tradeEntity.getCode() + "_member", JSONUtil.toJsonStr(tradeEntity), 3, TimeUnit.MINUTES);
        redisCache.setCacheObject(tradeEntity.getCode() + "_member" + "_promotionId", orderTradeDto.getPromotionId(), 3, TimeUnit.MINUTES);
//        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace().preCreate("订单主题：Mac笔记本", "LS123qwe123", "19999");
        //参照官方文档响应示例，解析返回结果
        String httpBodyStr = payResponse.getHttpBody();
        JSONObject jsonObject = JSONObject.parseObject(httpBodyStr);
        return jsonObject.getJSONObject("alipay_trade_precreate_response").get("qr_code").toString();
    }


    /**
     * 调用支付宝预下订单接口
     *
     * @param paymentAmount 充值金额
     * @return 二维码url
     * @throws Exception
     */
    public String pointsPay(Double paymentAmount) throws Exception {
        // 设置orderTrade信息
        OrderTrade tradeEntity = new OrderTrade();
        tradeEntity.setCode(UUID.randomUUID().toString(true).substring(0, 30));
        tradeEntity.setUserId(SecurityUtils.getUserId());
        tradeEntity.setProductId(-1);
        tradeEntity.setProductName("积分充值");
        tradeEntity.setUserName(SecurityUtils.getUsername());
        tradeEntity.setPaymentAmount(paymentAmount.intValue());

        //调用支付宝的接口
        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace()
                .preCreate(tradeEntity.getProductName(),
                        tradeEntity.getCode() + "_points",
                        tradeEntity.getPaymentAmount().toString());
        // 缓存到redis
        redisCache.setCacheObject(tradeEntity.getCode() + "_points", JSONUtil.toJsonStr(tradeEntity), 3, TimeUnit.MINUTES);
        //参照官方文档响应示例，解析返回结果
        String httpBodyStr = payResponse.getHttpBody();
        JSONObject jsonObject = JSONObject.parseObject(httpBodyStr);
        return jsonObject.getJSONObject("alipay_trade_precreate_response").get("qr_code").toString();
    }
}
